package remove_add_cascade;

import static org.junit.Assert.*;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * 
 * @author adam-bien.com
 *
 */
public class CascadeTest {

	private EntityManager em = null;
	private EntityTransaction tx = null;
	private EntityManagerFactory ef = null;
	@Before
	public void setUp() throws Exception {
		this.ef = Persistence.createEntityManagerFactory("test");
		this.em = ef.createEntityManager();
		this.tx = this.em.getTransaction();
		}
	
	@Test
	public void cascadeTest(){
		Master m1 = null;
		Slave s1 = null;
      
		//Create and persist the master
		tx.begin();
		m1 = new Master();
		em.persist(m1);
		tx.commit();
		
		//it should be known by the EM
		assertTrue(em.contains(m1));
		
		//persist the slave and manage the relations
		tx.begin();
		s1 = new Slave();
		
		//set the 1-N link
		m1.slaves.add(s1);
		
		//it's o.k. should be detached
		assertFalse(em.contains(s1));
		
		//set the backlink
		s1.master = m1;
		
		//and now remove the master again. The remove-operation should cascade
		em.remove(m1);
		
		//it's o.k. should be detached
		assertFalse(em.contains(s1));
		tx.commit();
		
		assertFalse(em.contains(m1));
		//s1 is still known. THIS ASSERTION FAILS
//		assertFalse(em.contains(s1));
		
		tx.begin();
		tx.commit();
		
		tx.begin();
		final Master m2 = new Master();
		em.persist(m2);
		tx.commit();	
	}
	
	@After
	public void tearDown(){
		if (em != null && em.getTransaction().isActive()) {
			em.getTransaction().rollback();
		}
		this.em.close();
		this.ef.close();
	}

}
